@@ -1,10 +1,11 @@
 import logging
+import time
 
 import re
 import socket
 from collections import OrderedDict
 
-import dicttoxml
+import dicttoxml2
 import xmltodict
 
 log = logging.getLogger(__name__)
@@ -38,6 +39,7 @@ class IAlarm(object):
         """
         self.host = host
         self.port = port
+        self.seq = 0
         self.sock = None
 
     def ensure_connection_is_open(self) -> None:
@@ -61,9 +63,7 @@ class IAlarm(object):
     def _send_request_list(self, xpath, command, offset=0, partial_list=None):
         if offset > 0:
             command['Offset'] = 'S32,0,0|%d' % offset
-        root_dict = self._create_root_dict(xpath, command)
-        self._send_dict(root_dict)
-        response = self._receive()
+        response = self._send_request_raw(xpath, command)
 
         if partial_list is None:
             partial_list = []
@@ -77,14 +77,25 @@ class IAlarm(object):
             # Continue getting elements increasing the offset
             self._send_request_list(xpath, command, offset, partial_list)
 
-        self._close_connection()
         return partial_list
 
-    def _send_request(self, xpath, command) -> dict:
+    def _send_request_raw(self, xpath, command) -> dict:
         root_dict = self._create_root_dict(xpath, command)
-        self._send_dict(root_dict)
-        response = self._receive()
-        self._close_connection()
+        try:
+            self._send_dict(root_dict)
+            response = self._receive()
+        except ConnectionError as err:
+            log.debug("Failed to send request, retrying with a new connection: %s", err)
+            self._close_connection()
+            # The alarm can start to drop responses if we send multiple requests back-to-back
+            time.sleep(1)
+            self._send_dict(root_dict)
+            response = self._receive()
+
+        return response
+
+    def _send_request(self, xpath, command) -> dict:
+        response = self._send_request_raw(xpath, command)
         return self._clean_response_dict(response, xpath)
 
     def get_mac(self) -> str:
@@ -109,7 +120,14 @@ class IAlarm(object):
             raise ConnectionError('An error occurred trying to connect to the alarm '
                                   'system or received an unexpected reply')
 
-    def get_status(self) -> int:
+    def get_status(self, include_memory_feature=False) -> int:
+        """
+        :param include_memory_feature:
+            .. deprecated:: 1.9.5
+                Memory feature is always enabled, this parameter has no effect
+        :return: The current alarm status
+        :raises ConnectionError:
+        """
         command = OrderedDict()
         command['DevStatus'] = None
         command['Err'] = None
@@ -122,22 +140,26 @@ class IAlarm(object):
         if status == -1:
             raise ConnectionError('Received an unexpected reply from the alarm')
 
-        zone_alarm = False
+        if status != self.ARMED_AWAY and status != self.ARMED_STAY:
+            # If the status is not "armed", there is no point in checking for "triggered"
+            return status
+
+        # The alarm can start to drop responses if we send multiple different requests back-to-back
+        time.sleep(1)
+
         command = OrderedDict()
         command['Total'] = None
         command['Offset'] = 'S32,0,0|0'
         command['Ln'] = None
         command['Err'] = None
         zone_status = self._send_request_list('/Root/Host/GetByWay', command)
+        self._close_connection()
         if zone_status is None:
             raise ConnectionError('An error occurred trying to connect to the alarm '
                                   'system')
         for zone in zone_status:
             if zone & self.ZONE_ALARM:
-                zone_alarm = True
-
-        if (status == self.ARMED_AWAY or status == self.ARMED_STAY) and zone_alarm:
-            return self.TRIGGERED
+                status = self.TRIGGERED
 
         return status
 
@@ -166,11 +188,13 @@ class IAlarm(object):
         self._send_request('/Root/Host/SetAlarmStatus', command)
 
     def _send_dict(self, root_dict) -> None:
-        xml = dicttoxml.dicttoxml(root_dict, attr_type=False, root=False)
+        xml = dicttoxml2.dicttoxml(root_dict, attr_type=False, root=False)
 
         self.ensure_connection_is_open()
 
-        msg = b'@ieM%04d00010000%s0001' % (len(xml), self._xor(xml))
+        self.seq += 1
+        msg = b'@ieM%04d%04d0000%s%04d' % (len(xml), self.seq, self._xor(xml),
+                                           self.seq)
         self.sock.send(msg)
 
     def _receive(self):
@@ -179,10 +203,20 @@ class IAlarm(object):
         except (socket.timeout, OSError, ConnectionRefusedError) as err:
             self.sock.close()
             raise ConnectionError("Connection error") from err
+
+        if not data:
+            self.sock.close()
+            raise ConnectionError("Connection error, received no reply")
+
         # It might happen to receive the err tag before the root, we just
         # remove it because it's not necessary
         decoded = self._xor(data[16:-4]).decode().replace("<Err>ERR|00</Err>",
                                                           "")
+
+        if not decoded:
+            self.sock.close()
+            raise ConnectionError("Connection error, received an unexpected reply")
+
         return xmltodict.parse(decoded, xml_attribs=False,
                                dict_constructor=dict,
                                postprocessor=self._xmlread)
@@ -244,3 +278,20 @@ class IAlarm(object):
             ki = i & 0x7f
             buf[i] = buf[i] ^ sz[ki]
         return buf
+
+
+if __name__ == "__main__":
+    ialarm = IAlarm("192.168.40.200")
+
+    # ialarm.disarm()
+
+    import time
+    for ii in range(30):
+        try:
+            print(ialarm.get_status())
+        except ConnectionError as e:
+            print(e)
+            print("no connection")
+            print(ialarm.sock)
+        time.sleep(15)
+        print("-----------")
